require "bitbucket/pr/post/version"
require 'json'
require 'optparse'
require 'io/console'


module Bitbucket
  module Pr
    class Post
      DOMAIN = 'bitbucket.org'
      PASSWORD_FILE = '~/.bitbucket_pr_post_password'
      def request(argv)
        set_opts argv
        confirm
        print_result post
      end

      private

      def set_opts(argv)
        @opts = {}
        o = OptionParser.new
        o.on('-p', '--pass=Bitbucket Password'){|v| @opts[:pass]  = v }
        o.on('-t', '--title=PR Title'){|v| @opts[:title] = v }
        o.on('-ps', '--public-slug=SLUG(Public Repository Name)'){|v| @opts[:public_slug]  = v }
        o.on('-ms', '--master-slug=SLUG(MasterRepository Name)'){|v| @opts[:master_slug]  = v }
        o.on('-u', '--user=Bitbucket User'){|v| @opts[:user]  = v }
        o.on('-D', '--debug'){|v| @opts[:debug] = v }
        o.on('-y', '--yes'){|v| @opts[:yes] = v }
        o.on('-o', '--open-url'){|v| @opts[:open_url] = true}
        o.on('-b', '--dst-branch=Destination Branch'){|v| @opts[:dst_branch] = v}
        o.on('-U', '--update'){|v| @opts[:update] = v}
        o.on('-d', '--description'){|v| @opts[:description] = v }

        o.parse!(argv)

        url = {}
        `git remote -v`.split("\n").each do |remote|
          if remote =~ /pub.*push/
            url[:pub] = remote.split(" ")[1]
          elsif remote =~ /origin.*push/
            url[:master] = remote.split(" ")[1]
          end
        end

        if url[:pub]
          dummy,domain,@pub_user,@public_slug = url[:pub].split /[@:\/]/
        end

        if url[:master]
          dummy,domain,@master_user,@master_slug = url[:master].split /[@:\/]/
          @dst_user = @master_user
        end

        if @opts[:pass]
          @pass = @opts[:pass]
        else
          fullpath = File.expand_path(PASSWORD_FILE)
          if File.exists? fullpath
            @pass = File.read(fullpath).strip
          else
            @pass = [(print "#{DOMAIN} Password ( or Make #{PASSWORD_FILE}): "), STDIN.noecho(&:gets).rstrip][1]
            puts
          end
        end

        @user = @pub_user || @master_user

        @user        = @opts[:user] || @user
        @public_slug = @opts[:pubilc_slug] || @public_slug.sub('.git', '')
        @master_slug = @opts[:master_slug] || @master_slug.sub('.git', '')
        @branch      = @opts[:branch] || `git branch | grep '*'`.split(" ")[1]
        @dst_branch  = @opts[:dst_branch] || 'master'

        if @branch == 'master'
          puts "Current Branch is master!"
          exit
        end

        if @opts[:title]
          @title = @opts[:title]
          @notice = ''
        else
          @title = `git log -1 --format="%s" --no-color --no-notes | head -1`.strip
          @title_notice = "\n             ^ This is Last Commit Mesasge. Use Option `--title` If You Want Change."
        end

        if @opts[:description]
          @description = @opts[:description]
        else
          @description = `git log -1 --format="%b" --no-color --no-notes`.strip
          @description_notice = "\n             ^ This is Last Commit Mesasge. Use Option `--description` If You Want Change."
        end

        debug_print @opts
      end

      def confirm
      puts <<-CONFIRM
      title: #{@title}#{@title_notice}
description: #{@description.gsub("\n", "\n             ")}#{@description_notice}
        src: #{@user}/#{@public_slug}/#{@branch}
             v v v
             v v v
        dst: #{@dst_user}/#{@master_slug}/#{@dst_branch}

      CONFIRM

        unless @opts[:yes]
          # Confirm
          input = [(print "#{method} PR to #{DOMAIN}? [Y/n] "), gets.rstrip][1]
          if input !~ /y(es)?/i or input == ''
            puts "canceled."
            exit
          end
        end
      end

      def request_data
        data = {
          title: @title,
          description: "#{@description}",
          source: {
            branch: { name: @branch },
            repository: { full_name: "#{@user}/#{@public_slug}" }
          },
          destination: {
            branch: { name: "master" },
          },
          "close_source_branch": true
        }

        debug_print(JSON.pretty_generate data)
        JSON.generate data
      end

      def method
        @opts[:update] ? 'PUT' : 'POST'
      end

      def mktemp
        if `uname`.include? 'Darwin'
          `/usr/bin/mktemp -t bitbucket-pr-post`.strip
        else
          `mktemp`.strip
        end
      end

      def post
        tempfile = mktemp
        File.write tempfile, request_data.gsub("\n", "\r\n")

        curl = %W{
          curl
          --silent
          -X #{method}
          -H "Content-Type: application/json"
          -u #{@user}:#{@pass}
          https://api.bitbucket.org/2.0/repositories/#{@dst_user}/#{@master_slug}/pullrequests/
          -d @#{tempfile}
        }
        cmd = curl.join " "
        debug_print cmd
        print 'send PR...'
        `#{cmd}`
      end

      def print_result(result)
        if result == ''
          puts "Nothing Responce. Maybe, already created pull request."
        else
          begin
            debug_print result
            result_json = JSON.load(result)
            debug_print result_json

            if e = result_json['error']
              print_response_error(e)
            else
              url = result_json['links']['html']['href']
              puts 'OK!'
              if @opts[:open_url]
                puts "Open Browser..."
                `open #{url}`
              else
                puts "URL: #{url}"
              end
            end
          rescue => e
            puts e.message
            puts "Responce is Not JSON ( or Broken ). Use Debug Option `-d`."
          end
        end
      end

      def print_response_error(e)
        puts "Response Error: #{e['message']}"
        if e['fields']
          e['fields'].each do|col, v|
            puts "  col"
            puts "    #{v.join(". ")}"
          end
        end
      end

      def debug_print(str)
        puts "DEBUG: #{str}" if @opts[:debug]
      end

    end
  end
end
